package speiger.src.coreengine.rendering.gui.animation;

import java.util.Objects;

import speiger.src.collections.objects.functions.consumer.ObjectFloatConsumer;
import speiger.src.collections.objects.maps.impl.misc.LinkedEnum2ObjectMap;
import speiger.src.collections.objects.maps.interfaces.Object2ObjectMap;
import speiger.src.collections.objects.maps.interfaces.Object2ObjectMap.Entry;
import speiger.src.collections.objects.utils.maps.Object2ObjectMaps;
import speiger.src.coreengine.rendering.gui.components.base.GuiComponent;
import speiger.src.coreengine.rendering.guiOld.helper.box.IGuiBox;

public class GuiAnimation {
	Object2ObjectMap<Target, IAction> actions;
	float duration;
	
	private GuiAnimation(Object2ObjectMap<Target, IAction> actions, float duration) {
		this.actions = actions;
		this.duration = duration;
	}
	
	public Builder copy() { return new Builder(this); }
	public static Builder of() { return new Builder(); }
	
	public float duration() { return duration; }
	public void apply(GuiComponent owner, float progress) {
		for(Entry<Target, IAction> entry : Object2ObjectMaps.fastIterable(actions)) {
			IAction action = entry.getValue();
			action.apply(entry.getKey(), owner, Math.min(progress, action.duration()));
		}
	}
	
	public static class Builder {
		Object2ObjectMap<Target, IAction> actions = new LinkedEnum2ObjectMap<>(Target.class);
		
		private Builder() {}
		private Builder(GuiAnimation owner) {
			this.actions.putAll(owner.actions);
		}
		
		public Builder add(Target target, IAction action) {
			this.actions.put(target, Objects.requireNonNull(action));
			return this;
		}
		
		public GuiAnimation build() {
			float duration = 0F;
			actions.values().mapToFloat(IAction::duration).reduce(Math::max);
			for(IAction action : actions.values()) {
				duration = Math.max(duration, action.duration());
			}
			return new GuiAnimation(actions, duration);
		}
	}
	
	public static enum Target {
		X(IGuiBox::setX),
		Y(IGuiBox::setY),
		WIDTH(IGuiBox::setWidth),
		HEIGHT(IGuiBox::setHeight),
		SCALE(IGuiBox::setScale);
		
		ObjectFloatConsumer<IGuiBox> consumer;
		
		private Target(ObjectFloatConsumer<IGuiBox> consumer) {
			this.consumer = consumer;
		}
		
		public void set(GuiComponent component, float value) {
			consumer.accept(component.getBox(), value);
		}
	}
}
