package speiger.src.coreengine.rendering.gui.animation;

import speiger.src.coreengine.rendering.gui.animation.GuiAnimation.Target;
import speiger.src.coreengine.rendering.gui.components.base.GuiComponent;

public interface IAction {
	public float duration();
	public void apply(Target target, GuiComponent component, float progress);
	public default IAction withPreDelay(float delay) { return this instanceof PreDelayedAction delayed ? new PreDelayedAction(delayed.action(), delay + delayed.delay()) : new PreDelayedAction(this, delay); }
	public default IAction withPostDelay(float delay) { return this instanceof PostDelayedAction delayed ? new PostDelayedAction(delayed.action(), delay + delayed.delay()) : new PostDelayedAction(this, delay); }
	public default IAction reverse() { return this instanceof ReversedAction reversed ? reversed.action() : new ReversedAction(this); }
	
	public record PreDelayedAction(IAction action, float delay) implements IAction {
		@Override
		public float duration() { return delay + action.duration(); }
		@Override
		public void apply(Target target, GuiComponent component, float progress) {
			action.apply(target, component, Math.max(progress - delay, 0F));
		}
	}
	
	public record PostDelayedAction(IAction action, float delay) implements IAction {
		@Override
		public float duration() { return action.duration() + delay; }
		@Override
		public void apply(Target target, GuiComponent component, float progress) {
			action.apply(target, component, Math.min(progress, action.duration()));
		}
	}
	
	public record ReversedAction(IAction action) implements IAction {
		@Override
		public float duration() { return action.duration(); }
		@Override
		public void apply(Target target, GuiComponent component, float progress) {
			action.apply(target, component, action.duration() - progress);
		}
	}
	
	
}